"""
- `UNAUTHORISED_REQUEST_ERROR_MESSAGE` (str): Plain text error message. Defaults to "Unauthorised request. {0}".
- `UNAUTHORISED_REQUEST_ERROR_TEMPLATE` (str): Path to 401 template that should be rendered in case of 401
  responses. Defaults to empty string (not provided).
- `AUTH_USER` (str): Default ``auth_user`` for ``ska.sign_url`` function. Defaults to "ska-auth-user".
- `USER_GET_CALLBACK` (str): User get callback (when user is fetched in auth backend).
- `USER_CREATE_CALLBACK` (str): User create callback (when user is created in auth backend).
- `USER_INFO_CALLBACK` (str): User info callback.
- `REDIRECT_AFTER_LOGIN` (str): Redirect after login.
- `DB_STORE_SIGNATURES` (bool): If set to True, signatures are stored in the database.
- `DB_PERFORM_SIGNATURE_CHECK` (bool): If set to True, an extra check is fired on whether the token has
  already been used or not.
- `PROVIDERS` (dict): A dictionary where key is the provider UID and the key is another dictionary holding
  the following provider specific keys: 'SECRET_KEY', 'USER_GET_CALLBACK', 'USER_CREATE_CALLBACK',
  'USER_INFO_CALLBACK', 'REDIRECT_AFTER_LOGIN'. Note, that the 'SECRET_KEY' is a required key. The rest
  are optional, and if given, override respectively the values of ``ska.contrib.django.ska.settings``.
"""

__title__ = 'ska.contrib.django.ska.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'AUTH_USER', 'UNAUTHORISED_REQUEST_ERROR_MESSAGE', 'UNAUTHORISED_REQUEST_ERROR_TEMPLATE',
    'USER_GET_CALLBACK', 'USER_CREATE_CALLBACK', 'USER_INFO_CALLBACK', 'REDIRECT_AFTER_LOGIN',
    'DB_STORE_SIGNATURES', 'DB_PERFORM_SIGNATURE_CHECK', 'PROVIDERS'
)

ugettext = lambda s: s

AUTH_USER = 'ska-auth-user'

UNAUTHORISED_REQUEST_ERROR_MESSAGE = ugettext("Unauthorised request. {0}")

UNAUTHORISED_REQUEST_ERROR_TEMPLATE = ''

USER_GET_CALLBACK = None
USER_CREATE_CALLBACK = None
USER_INFO_CALLBACK = None
REDIRECT_AFTER_LOGIN = ''

DB_STORE_SIGNATURES = False
DB_PERFORM_SIGNATURE_CHECK = False

PROVIDERS = {}


__title__ = 'ska.helpers'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'get_callback_func', 'dict_keys', 'dict_to_ordered_list', 'sorted_urlencode', 'extract_signed_data'
    )

from six import PY3

try:
    from six.moves.urllib.parse import quote
except ImportError as e:
    if PY3:
        from urllib.parse import quote
    else:
        from urllib import quote

def get_callback_func(function):
    """
    Takes a string and tries to extract a function from it.

    :param mixed function: If `callable` is given, return as is. If `string` is given, try to
        extract the function from the string given and return.
    :return callable: Returns `callable` if what's extracted is callable or None otherwise.
    """
    if callable(function):
        return function
    elif isinstance(function, str):
        path = function.split('.')
        try:
            exec('from %s import %s as %s' % ('.'.join(path[0:-1]), path[-1], 'func'))
            if callable(func):
                return func
        except:
            return None

def dict_keys(data, return_string=False):
    """
    Gets sorted keys from dictionary given. If ``return_string`` argument is set to True,
    returns keys joined by commas.

    :param dict data:
    :param bool return_string:
    """
    keys = list(data.keys())
    keys.sort()

    if return_string:
        return ','.join(keys)

    return keys

def dict_to_ordered_list(data):
    """
    Gets extra as ordered list. Actually, I'm not sure whether I should or should not
    be using ``ordereddict`` here.

    :param dict data:
    :return list:
    """
    items = list(data.items())
    items.sort()
    return items

def sorted_urlencode(data, quoted=True):
    """
    Similar to built-in ``urlencode``, but always puts data in a sorted constant way that
    stays the same between varios python versions.
    """
    l = ["{0}={1}".format(k, v) for k, v in dict_to_ordered_list(data)]
    res = '&'.join(l)
    if quoted:
        res = quote(res)
    return res

def extract_signed_data(data, extra):
    """
    Filters out non-white-listed items from the ``extra`` dictionary given.

    :param dict data:
    :param list extra:
    :return dict:
    """
    extracted_extra = dict(data)
    for key, value in data.items():
        if not key in extra:
            extracted_extra.pop(key)

    return extracted_extra



__title__ = 'ska.exceptions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('BaseException', 'ImproperlyConfigured', 'InvalidData')


class BaseException(Exception):
    """
    Base exception.
    """


class ImproperlyConfigured(BaseException):
    """
    Exception raised when developer didn't configure/write the code properly.
    """


class InvalidData(BaseException):
    """
    Raised when invalid data (tumpered) is detected.
    """



from __future__ import absolute_import

"""
- ``validate_signed_request``: Function decorator. Validate request signature. Applies appropriate validation
  mechanism to the request data. Assumes ``SKA_SECRET_KEY`` to be in ``settings`` module.

  Arguments to be used with `ska.validate_signed_request_data` shortcut function.

  :param str secret_key: The shared secret key.
  :param str signature_param: Name of the (for example GET or POST) param name which holds
      the ``signature`` value.
  :param str auth_user_param: Name of the (for example GET or POST) param name which holds
      the ``auth_user`` value.
  :param str valid_until_param: Name of the (foe example GET or POST) param name which holds
      the ``valid_until`` value.

- ``sign_url``: Method decorator (to be used in models). Signs the URL.

  Arguments to be used with `ska.sign_url` shortcut function.

  :param str auth_user: Username of the user making the request.
  :param str secret_key: The shared secret key.
  :param float|str valid_until: Unix timestamp. If not given, generated automatically (now + lifetime).
  :param int lifetime: Signature lifetime in seconds.
  :param str suffix: Suffix to add after the ``endpoint_url`` and before the appended signature params.
  :param str signature_param: Name of the GET param name which would hold the generated signature value.
  :param str auth_user_param: Name of the GET param name which would hold the ``auth_user`` value.
  :param str valid_until_param: Name of the GET param name which would hold the ``valid_until`` value.
"""

__title__ = 'ska.contrib.django.ska.decorators'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('validate_signed_request', 'sign_url')

from six import PY3, text_type

from ska import validate_signed_request_data, sign_url as ska_sign_url
from ska.defaults import (
    SIGNATURE_LIFETIME, DEFAULT_URL_SUFFIX, DEFAULT_SIGNATURE_PARAM, DEFAULT_AUTH_USER_PARAM,
    DEFAULT_VALID_UNTIL_PARAM, DEFAULT_EXTRA_PARAM
    )
from ska.contrib.django.ska.settings import (
    SECRET_KEY, AUTH_USER, UNAUTHORISED_REQUEST_ERROR_MESSAGE,
    UNAUTHORISED_REQUEST_ERROR_TEMPLATE
    )
from ska.contrib.django.ska.http import HttpResponseUnauthorized

from django.utils.translation import ugettext_lazy as _
from django.shortcuts import render

class BaseValidateSignedRequest(object):
    def __init__(self, secret_key=SECRET_KEY, signature_param=DEFAULT_SIGNATURE_PARAM, \
                 auth_user_param=DEFAULT_AUTH_USER_PARAM, valid_until_param=DEFAULT_VALID_UNTIL_PARAM, \
                 extra_param=DEFAULT_EXTRA_PARAM):
        self.secret_key = secret_key
        self.signature_param = signature_param
        self.auth_user_param = auth_user_param
        self.valid_until_param = valid_until_param
        self.extra_param = extra_param

class ValidateSignedRequest(BaseValidateSignedRequest):
    """
    Function decorator. Validate request signature. Applies appropriate validation mechanism to the request
    data. Assumes ``SKA_SECRET_KEY`` to be in ``settings`` module.

    Arguments to be used with `ska.validate_signed_request_data` shortcut function.

    :attribute str secret_key: The shared secret key.
    :attribute str signature_param: Name of the (for example GET or POST) param name which holds
        the ``signature`` value.
    :attribute str auth_user_param: Name of the (for example GET or POST) param name which holds
        the ``auth_user`` value.
    :attribute str valid_until_param: Name of the (foe example GET or POST) param name which holds
        the ``valid_until`` value.
    :attribute str extra_param: Name of the (foe example GET or POST) param name which holds
        the ``extra`` value.

    :example:

    >>> from ska.contrib.django.ska.decorators import validate_signed_request
    >>> 
    >>> @validate_signed_request()
    >>> def detail(request, slug, template_name='foo/detail.html'):
    >>>     # Your code
    """
    def __call__(self, func):
        def inner(request, *args, **kwargs):
            # Validating the request.
            validation_result = validate_signed_request_data(
                data = request.REQUEST,
                secret_key = self.secret_key,
                signature_param = self.signature_param,
                auth_user_param = self.auth_user_param,
                valid_until_param = self.valid_until_param,
                extra_param = self.extra_param
                )
            if validation_result.result is True:
                # If validated, just return the func as is.
                return func(request, *args, **kwargs)
            else:
                # Otherwise...
                if UNAUTHORISED_REQUEST_ERROR_TEMPLATE:
                    # If template to display the error message is set in ska (django-ska) settings,
                    # use it to render the message and return ``HttpResponseUnauthorized`` response
                    # describing the error.
                    response_content = render(
                        request,
                        UNAUTHORISED_REQUEST_ERROR_TEMPLATE,
                        {'reason': '; '.join(validation_result.reason)}
                        )
                    return HttpResponseUnauthorized(response_content)
                else:
                    # Otherwise, return plain text message with describing the error.
                    return HttpResponseUnauthorized(
                        _(UNAUTHORISED_REQUEST_ERROR_MESSAGE).format('; '.join(validation_result.reason))
                        )
        return inner

validate_signed_request = ValidateSignedRequest


class MethodValidateSignedRequest(BaseValidateSignedRequest):
    """
    Method decorator. Validate request signature. Applies appropriate validation mechanism to the request
    data. Assumes ``SKA_SECRET_KEY`` to be in ``settings`` module.

    Arguments to be used with `ska.validate_signed_request_data` shortcut function.

    :attribute str secret_key: The shared secret key.
    :attribute str signature_param: Name of the (for example GET or POST) param name which holds
        the ``signature`` value.
    :attribute str auth_user_param: Name of the (for example GET or POST) param name which holds
        the ``auth_user`` value.
    :attribute str valid_until_param: Name of the (foe example GET or POST) param name which holds
        the ``valid_until`` value.
    :attribute str extra_param: Name of the (foe example GET or POST) param name which holds
        the ``extra`` value.

    :example:

    >>> from ska.contrib.django.ska.decorators import m_validate_signed_request
    >>> 
    >>> class FooDetailView(View):
    >>>     @validate_signed_request()
    >>>     def get(self, request, slug, template_name='foo/detail.html'):
    >>>         # Your code
    """
    def __call__(self, func):
        def inner(this, request, *args, **kwargs):
            # Validating the request.
            validation_result = validate_signed_request_data(
                data = request.REQUEST,
                secret_key = self.secret_key,
                signature_param = self.signature_param,
                auth_user_param = self.auth_user_param,
                valid_until_param = self.valid_until_param,
                extra_param = self.extra_param
                )
            if validation_result.result is True:
                # If validated, just return the func as is.
                return func(this, request, *args, **kwargs)
            else:
                # Otherwise...
                if UNAUTHORISED_REQUEST_ERROR_TEMPLATE:
                    # If template to display the error message is set in ska (django-ska) settings,
                    # use it to render the message and return ``HttpResponseUnauthorized`` response
                    # describing the error.
                    response_content = render(
                        request,
                        UNAUTHORISED_REQUEST_ERROR_TEMPLATE,
                        {'reason': '; '.join(validation_result.reason)}
                        )
                    return HttpResponseUnauthorized(response_content)
                else:
                    # Otherwise, return plain text message with describing the error.
                    return HttpResponseUnauthorized(
                        _(UNAUTHORISED_REQUEST_ERROR_MESSAGE).format('; '.join(validation_result.reason))
                        )
        return inner

m_validate_signed_request = MethodValidateSignedRequest


class SignAbsoluteURL(object):
    """
    Method decorator (to be used in models). Signs the URL.

    Arguments to be used with `ska.sign_url` shortcut function.

    :attribute str auth_user: Username of the user making the request.
    :attribute str secret_key: The shared secret key.
    :attribute float|str valid_until: Unix timestamp. If not given, generated automatically (now + lifetime).
    :attribute int lifetime: Signature lifetime in seconds.
    :attribute str suffix: Suffix to add after the ``endpoint_url`` and before the appended signature params.
    :attribute str signature_param: Name of the GET param name which would hold the generated signature value.
    :attribute str auth_user_param: Name of the GET param name which would hold the ``auth_user`` value.
    :attribute str valid_until_param: Name of the GET param name which would hold the ``valid_until`` value.
    :attribute dict extra: Dict of extra params to append to signed URL.
    :attribute str extra_param: Name of the GET param name which would hold the ``extra`` value.

    :example:

    >>> from ska.contrib.django.ska.decorators import sign_url
    >>> 
    >>> class FooItem(models.Model):
    >>>     title = models.CharField(_("Title"), max_length=100)
    >>>     slug = models.SlugField(unique=True, verbose_name=_("Slug"))
    >>>     body = models.TextField(_("Body"))
    >>>
    >>>     @sign_url()
    >>>     def get_signed_absolute_url(self):
    >>>         return reverse('foo.detail', kwargs={'slug': self.slug})
    """
    def __init__(self, auth_user=AUTH_USER, secret_key=SECRET_KEY, valid_until=None, lifetime=SIGNATURE_LIFETIME, \
                 suffix=DEFAULT_URL_SUFFIX, signature_param=DEFAULT_SIGNATURE_PARAM, \
                 auth_user_param=DEFAULT_AUTH_USER_PARAM, valid_until_param=DEFAULT_VALID_UNTIL_PARAM, \
                 extra={}, extra_param=DEFAULT_EXTRA_PARAM):
        self.auth_user = auth_user
        self.secret_key = secret_key
        self.valid_until = valid_until
        self.lifetime = lifetime
        self.suffix = suffix
        self.signature_param = signature_param
        self.auth_user_param = auth_user_param
        self.valid_until_param = valid_until_param
        self.extra = extra
        self.extra_param = extra_param

    def __call__(self, func):
        def inner(this, *args, **kwargs):
            if not PY3:
                url = text_type(func(this, *args, **kwargs))
            else:
                url = func(this, *args, **kwargs)

            return ska_sign_url(
                auth_user = self.auth_user,
                secret_key = self.secret_key,
                valid_until = self.valid_until,
                lifetime = self.lifetime,
                url = url,
                suffix = self.suffix,
                signature_param = self.signature_param,
                auth_user_param = self.auth_user_param,
                valid_until_param = self.valid_until_param,
                extra = self.extra,
                extra_param = self.extra_param
                )
        return inner

sign_url = SignAbsoluteURL



__title__ = 'ska.contrib.django.ska.utils'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('purge_signature_data', 'get_secret_key', 'get_provider_data')

import datetime

from ska.defaults import DEFAULT_PROVIDER_PARAM
from ska.contrib.django.ska.models import Signature
from ska.contrib.django.ska.settings import PROVIDERS, SECRET_KEY

def purge_signature_data():
    """
    Purges old signature data (valid_until < now).
    """
    Signature._default_manager.filter(valid_until__lt=datetime.datetime.now()).delete()

def get_secret_key(data, default=SECRET_KEY):
    """
    Obtain the secret key from request data given. This happens by looking up the secret key
    by `provider` param from the request data in the dictionary of ``PROVIDERS`` defined in
    settings module. If not found, fall back to the ``default`` value given, which is by
    default the globally set secret key.

    :param dict data:
    :param string default: Secret key value to be used as default. By default, the globally set
        secret key is used.
    """
    provider = data.get(DEFAULT_PROVIDER_PARAM, None)
    if provider:
        provider_data = PROVIDERS.get(provider, None)
        if provider_data:
            return provider_data.get('SECRET_KEY', default)

    return default

def get_provider_data(data):
    """
    Obtain the secret key from request data given. This happens by looking up the secret key
    by `provider` param from the request data in the dictionary of ``PROVIDERS`` defined in
    settings module. If not found, fall back to the ``default`` value given, which is by
    default the globally set secret key.

    :param dict data:
    :param string default: Secret key value to be used as default. By default, the globally set
        secret key is used.
    """
    provider = data.get(DEFAULT_PROVIDER_PARAM, None)
    if provider:
        return PROVIDERS.get(provider, None)
    return {}



__title__ = 'ska.contrib.django.ska.urls'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('urlpatterns',)

from django.conf.urls import patterns, url

from ska.contrib.django.ska.views import login

urlpatterns = patterns('',
    url(r'^login/', login, name="ska.login"),
)



from __future__ import print_function

__title__ = 'ska.contrib.django.ska.tests'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

import unittest
import os

PROJECT_DIR = lambda base : os.path.join(os.path.dirname(__file__), base).replace('\\','/')

SKA_TEST_USER_USERNAME = 'test_admin'
SKA_TEST_USER_PASSWORD = 'test'
PRINT_INFO = True

def print_info(func):
    """
    Prints some useful info.
    """
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        result = func(self, *args, **kwargs)

        print('\n\n%s' % func.__name__)
        print('============================')
        if func.__doc__:
            print('""" %s """' % func.__doc__.strip())
        print('----------------------------')
        if result is not None:
            print(result)
        print('\n++++++++++++++++++++++++++++')

        return result
    return inner

# *********************************************************************
# *********************************************************************
# *********************************************************************

import random

from six import PY3

try:
    from six.moves import range as xrange
except ImportError:
    if PY3:
        xrange = range

if not PY3:
    from string import translate, maketrans, punctuation
else:
    from string import punctuation

import radar

FACTORY = """
    Sed dictum in tellus non iaculis. Aenean ac interdum ipsum. Etiam tempor quis ante vel rhoncus. Nulla
    facilisi. Curabitur iaculis consequat odio ut imperdiet? Integer accumsan; nisl vitae fermentum malesuada,
    sapien nulla sodales orci, et elementum lacus purus vel purus! Nullam orci neque, tristique in porta id,
    pretium ac sem. Fusce non est risus. Fusce convallis tellus augue, quis volutpat tellus dapibus sagittis.
    Integer lacinia commodo risus vel cursus. Etiam vitae dui in dolor porta luctus sed id elit. Nulla et est
    nec magna facilisis sagittis. Praesent tincidunt dictum lectus, sed aliquam eros. Donec placerat tortor ut
    lorem facilisis congue. Quisque ac risus nibh. Etiam ultrices nibh justo; sed mollis ipsum dapibus vitae.Ut
    vitae molestie erat. Mauris ac justo quis ante posuere vehicula. Vivamus accumsan mi volutpat diam lacinia,
    vitae semper lectus pharetra. Cras ultrices arcu nec viverra consectetur. Cras placerat ante quis dui
    consequat cursus. Nulla at enim dictum, consectetur ligula eget, vehicula nisi. Suspendisse eu ligula vitae
    est tristique accumsan nec adipiscing risus.Donec tempus dui eget mollis fringilla. Fusce eleifend lacus lectus,
    vel ornare felis lacinia ut. Morbi vel adipiscing augue. Vestibulum ante ipsum primis in faucibus orci luctus et
    ultrices posuere cubilia Curae; Cras mattis pulvinar lacus, vitae pulvinar magna egestas non. Aliquam in urna
    quis leo feugiat faucibus. Aliquam erat volutpat. Maecenas non mauris libero. Suspendisse nisi lorem, cursus a
    tristique a, porttitor in nisl. Mauris pellentesque gravida mi non mattis. Cras mauris ligula, interdum semper
    tincidunt sed, ornare a ipsum. Nulla ultrices tempus tortor vitae vehicula.Etiam at augue suscipit, vehicula
    sapien sit amet; eleifend orci. Etiam venenatis leo nec cursus mattis. Nulla suscipit nec lorem et lobortis.
    Donec interdum vehicula massa sed aliquam. Praesent eleifend mi sed mi pretium pellentesque. In in nisi tincidunt,
    commodo lorem quis; tincidunt nisl. In suscipit quam a vehicula tincidunt! Fusce vitae varius nunc. Proin at
    ipsum ac tellus hendrerit ultricies. Phasellus auctor hendrerit sapien viverra facilisis. Suspendisse lacus erat,
    cursus at dolor in, vulputate convallis sapien. Etiam augue nunc, lobortis vel viverra sit amet, pretium et
    lacus.Pellentesque elementum lectus eget massa tempus elementum? Nulla nec auctor dolor. Aliquam congue purus
    quis libero fermentum cursus. Etiam quis massa ac nisl accumsan convallis vitae ac augue. Mauris neque est,
    posuere quis dolor non, volutpat gravida tortor. Cum sociis natoque penatibus et magnis dis parturient montes,
    nascetur ridiculus mus. Vivamus ullamcorper, urna at ultrices aliquam, orci libero gravida ligula, non pulvinar
    sem magna sed tortor. Sed elementum leo viverra ipsum aliquet convallis. Suspendisse scelerisque auctor sapien.
    Mauris enim nisl, sollicitudin at rhoncus vitae, convallis nec mauris. Phasellus sollicitudin dui ut luctus
    consectetur. Vivamus placerat, neque id sagittis porttitor, nunc quam varius dolor, sit amet egestas nulla
    risus eu odio. Mauris gravida eleifend laoreet. Aenean a nulla nisl. Integer pharetra magna adipiscing, imperdiet
    augue ac, blandit felis. Cras id aliquam neque, vel consequat sapien.Duis eget vulputate ligula. Aliquam ornare
    dui non nunc laoreet, non viverra dolor semper.
    """

if not PY3:
    split_words = lambda f: list(set(translate(f.lower(), maketrans(punctuation, ' ' * len(punctuation))).split()))
else:
    split_words = lambda f: list(set(f.lower().translate(str.maketrans("", "", punctuation)).split()))

split_sentences = lambda f: f.split('?')
change_date = lambda: bool(random.randint(0, 1))

WORDS = split_words(FACTORY)
SENTENCES = split_sentences(FACTORY)
NUM_ITEMS = 5

_ = lambda s: s

# *********************************************************************
# *********************************************************************
# *********************************************************************

# Skipping from non-Django tests.
if os.environ.get("DJANGO_SETTINGS_MODULE", None):

    from django.test import Client
    from django.utils.text import slugify
    from django.contrib.auth.models import User

    from ska import sign_url
    from ska.defaults import DEFAULT_PROVIDER_PARAM
    from ska.contrib.django.ska.settings import (
        SECRET_KEY, DB_STORE_SIGNATURES, DB_PERFORM_SIGNATURE_CHECK, PROVIDERS
        )

    from foo.models import FooItem

# *********************************************************************
# *********************************************************************
# *********************************************************************

    def create_admin_user():
        """
        Create a user for testing the dashboard.

        TODO: At the moment an admin account is being tested. Automated tests with diverse accounts are
        to be implemented.
        """
        u = User()
        u.username = SKA_TEST_USER_USERNAME
        u.email = 'admin@dev.django-ska.com'
        u.is_superuser = True
        u.is_staff = True
        u.set_password(SKA_TEST_USER_PASSWORD)

        try:
            u.save()
        except Exception as e:
            pass

    def generate_data(num_items=NUM_ITEMS):
        words = WORDS[:]

        random_date = radar.random_datetime()

        for index in xrange(num_items):
            # Saving an item to database
            i = FooItem()
            random_name = words[random.randint(0, len(words) - 1)]

            if not PY3:
                i.title = unicode(random_name).capitalize()
                i.body = unicode(SENTENCES[random.randint(0, len(SENTENCES) - 1)])
            else:
                i.title = str(random_name).capitalize()
                i.body = str(SENTENCES[random.randint(0, len(SENTENCES) - 1)])

            i.slug = slugify(i.title)
            random_date = radar.random_datetime() if change_date() else random_date
            i.date_published = random_date

            try:
                i.save()
                words.remove(random_name)

                if 0 == len(words):
                    words = WORDS

            except Exception as e:
                print(e)

    # *********************************************************************
    # *********************************************************************
    # *********************************************************************

    class SkaDecoratorsTest(unittest.TestCase):
        """
        Testing model- and view- decorators.
        """
        def setUp(self):
            generate_data()

            # Testing the URLs
            self.item = FooItem._default_manager.all()[0]

        @print_info
        def test_01_model_decorator(self):
            """
            Test the ``ska.contrib.django.ska.decorators.sign_url`` model decorator.
            """
            # Testing signed URLs
            signed_absolute_url = self.item.get_signed_absolute_url()
            self.assertTrue(signed_absolute_url is not None)
            return signed_absolute_url

        @print_info
        def test_02_view_decorator_with_signed_url(self):
            """
            Test the ``ska.contrib.django.ska.decorators.validate_signed_request`` view decorator with signed URL.
            """
            flow = []

            # Testing signed URLs
            signed_absolute_url = self.item.get_signed_absolute_url()
            self.assertTrue(signed_absolute_url is not None)
            flow.append(('Signed absolute URL', signed_absolute_url))

            # Testing view with signed URL
            c = Client()
            response = c.get(signed_absolute_url, {})
            self.assertTrue(response.status_code in (200, 201, 202))
            flow.append(('Response status code for signed URL', response.status_code))

            return flow

        @print_info
        def test_03_view_decorator_with_unsigned_url(self):
            """
            Test the ``ska.contrib.django.ska.decorators.validate_signed_request`` view decorator with unsigned URL.
            """
            flow = []

            # Testing unsigned URLs
            absolute_url = self.item.get_absolute_url()
            self.assertTrue(absolute_url is not None)
            flow.append(('Unsigned absolute URL', absolute_url))

            # Testing view with signed URL
            c = Client()
            response = c.get(absolute_url, {})
            self.assertTrue(response.status_code in (401,))
            flow.append(('Response status code for unsigned URL', response.status_code))
            flow.append(('Response content for unsigned URL', response.content))

            return flow

        @print_info
        def test_04_extra(self):
            """
            Testing extra dict.
            """
            #TODO


    class SkaAuthenticationBackendTest(unittest.TestCase):
        """
        Tests for authentication backends.
        """
        def setUp(self):
            self.AUTH_USER = 'test_auth_backend_user'
            self.AUTH_USER_EMAIL = 'test_ska_auth_user@mail.example.com'
            self.AUTH_USER_FIRST_NAME = 'John'
            self.AUTH_USER_LAST_NAME = 'Doe'
            self.PROVIDER_NAME = 'client_1.admins'
            self.LOGIN_URL = '/ska/login/'

        def __test_login(self, secret_key, response_codes, provider_name=None):
            flow = []

            first_response_code, second_response_code = response_codes

            # Testing signed URLs
            extra = {
                'email': self.AUTH_USER_EMAIL,
                'first_name': self.AUTH_USER_FIRST_NAME,
                'last_name': self.AUTH_USER_LAST_NAME,
            }
            if provider_name:
                extra.update({DEFAULT_PROVIDER_PARAM: provider_name})

            signed_login_url = sign_url(
                auth_user = self.AUTH_USER,
                secret_key = secret_key,
                url = self.LOGIN_URL,
                extra = extra
                )

            self.assertTrue(signed_login_url is not None)
            flow.append(('Signed login URL', signed_login_url))

            # Testing view with signed URL
            c = Client()
            response = c.get(signed_login_url, {})
            self.assertTrue(response.status_code in (first_response_code,))
            flow.append(('Response status code for signed URL', response.status_code))

            if DB_STORE_SIGNATURES and DB_PERFORM_SIGNATURE_CHECK:
                # Testing again with signed URL and this time, it should fail
                c = Client()
                response = c.get(signed_login_url, {})
                self.assertTrue(response.status_code in (second_response_code,))
                flow.append(('Response status code for signed URL', response.status_code))

            return flow

        @print_info
        def test_01_login(self):
            """
            Test authentication using general ``SECRET_KEY``.
            """
            return self.__test_login(SECRET_KEY, [302, 403])

        @print_info
        def test_02_provider_login(self):
            """
            Test authentication using ``SECRET_KEY`` defined in ``PROVIDERS``.
            """
            secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
            return self.__test_login(secret_key, [302, 403], self.PROVIDER_NAME)

        @print_info
        def test_03_login_fail_wrong_secret_key(self):
            """
            Fail test authentication using general ``SECRET_KEY``, providing
            wrong secret key.
            """
            return self.__test_login(SECRET_KEY + 'wrong', [403, 403])

        @print_info
        def test_04_provider_login_fail_wrong_secret_key(self):
            """
            Fail test authentication using ``SECRET_KEY`` defined in ``PROVIDERS``, providing
            wrong secret key.
            """
            secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
            return self.__test_login(secret_key + 'wrong', [403, 403], self.PROVIDER_NAME)

        @print_info
        def test_05_provider_login_fail_wrong_provider(self):
            """
            Fail test authentication using ``SECRET_KEY`` defined in ``PROVIDERS``, providing
            wrong provider name.
            """
            secret_key = PROVIDERS[self.PROVIDER_NAME]['SECRET_KEY']
            return self.__test_login(secret_key + 'wrong', [403, 403], self.PROVIDER_NAME + 'wrong')


if __name__ == "__main__":
    # Tests
    unittest.main()



from __future__ import print_function

__title__ = 'ska.generate_signed_url'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('main',)

import argparse

from ska import sign_url

def main():
    """
    Prints signed URL to console.

    :example:

    $ python src/ska/generate_signature.py -au user -sk test
    """
    parser = argparse.ArgumentParser(description="""
    Generates signed URL.
    """)

    parser.add_argument("-au", "--auth-user", dest="auth_user", type=str, help="`auth_user` value", \
                        metavar="AUTH_USER")
    parser.add_argument("-sk", "--secret-key", dest="secret_key", type=str, help="`secret_key` value", \
                        metavar="SECRET_KEY")
    parser.add_argument("-vu", "--valid-until", dest="valid_until", type=float, help="`valid_until` value", \
                        metavar="VALID_UNTIL")
    parser.add_argument("-l", "--lifetime", dest="lifetime", type=int, help="`lifetime` value", metavar="LIFETIME")
    parser.add_argument("-u", "--url", dest="url", type=str, help="URL to sign", metavar="URL")
    parser.add_argument("-s", "--suffix", dest="suffix", type=str, help="Added after the `url`.", metavar="SUFFIX")
    parser.add_argument("-sp", "--signature-param", dest="signature_param", type=str, \
                        help="GET param holding the `signature` value", metavar="SIGNATURE_PARAM")
    parser.add_argument("-aup", "--auth-user-param", dest="auth_user_param", type=str, \
                        help="GET param holding the `auth_user` value", metavar="AUTH_USER_PARAM")
    parser.add_argument("-vup", "--valid-until-param", dest="valid_until_param", type=str, \
                        help="GET param holding the `auth_user` value", metavar="VALID_UNTIL_PARAM")

    args = parser.parse_args()

    kwargs = {}

    if args.auth_user:
        kwargs.update({'auth_user': args.auth_user})

    if args.secret_key:
        kwargs.update({'secret_key': args.secret_key})

    if args.valid_until:
        kwargs.update({'valid_until': args.valid_until})

    if args.lifetime:
        kwargs.update({'lifetime': args.lifetime})

    if args.url:
        kwargs.update({'url': args.url})

    if args.suffix:
        kwargs.update({'suffix': args.suffix})

    if args.signature_param:
        kwargs.update({'signature_param': args.signature_param})

    if args.auth_user_param:
        kwargs.update({'auth_user_param': args.auth_user_param})

    if args.valid_until_param:
        kwargs.update({'valid_until_param': args.valid_until_param})

    try:
        signed_url = sign_url(**kwargs)
        print(signed_url)
    except Exception as e:
        print(e)

if __name__ == "__main__":
    main()



from __future__ import print_function

__title__ = 'ska.tests'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

import unittest
import datetime
from copy import copy

from six import PY3, text_type

try:
    from six.moves.urllib.parse import urlparse, parse_qs
except ImportError as e:
    if PY3:
        from urllib.parse import urlparse, parse_qs
    else:
        from urlparse import urlparse, parse_qs

from ska import Signature, RequestHelper, TIMESTAMP_FORMAT
from ska import sign_url, validate_signed_request_data, signature_to_dict
from ska import error_codes

PRINT_INFO = True
TRACK_TIME = False

def print_info(func):
    """
    Prints some useful info.
    """
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        if TRACK_TIME:
            import simple_timer
            timer = simple_timer.Timer() # Start timer

        result = func(self, *args, **kwargs)

        if TRACK_TIME:
            timer.stop() # Stop timer

        print('\n\n%s' % func.__name__)
        print('============================')
        if func.__doc__:
            print('""" %s """' % func.__doc__.strip())
        print('----------------------------')
        if result is not None:
            print(result)
        if TRACK_TIME:
            print('done in %s seconds' % timer.duration)
        print('\n++++++++++++++++++++++++++++')

        return result
    return inner

def timestap_to_human_readable(timestamp):
    """
    Converts Unix timestamp to human readable string.

    :param float:
    :return str:
    """
    dt = datetime.datetime.fromtimestamp(float(timestamp))
    return dt.strftime(TIMESTAMP_FORMAT)

class SignatureTest(unittest.TestCase):
    """
    Tests of ``ska.Signature`` class.
    """
    def setUp(self):
        self.auth_user = 'user'
        self.secret_key = 'secret'

    @print_info
    def test_01_signature_test(self):
        """
        Signature test.
        """
        flow = []

        # Generate signature
        sig = Signature.generate_signature(
            auth_user = self.auth_user,
            secret_key = self.secret_key
            )

        flow.append(('Valid until used', sig.valid_until))
        flow.append(('Valid until (human readable)', timestap_to_human_readable(sig.valid_until)))
        flow.append(('Signature generated', sig.signature))
        flow.append(('Signature is expired', sig.is_expired()))

        # Check if not expired
        self.assertTrue(not sig.is_expired())

        # Check if valid
        validation_result = Signature.validate_signature(
                signature = sig.signature,
                auth_user = self.auth_user,
                secret_key = self.secret_key,
                valid_until = sig.valid_until,
                return_object = True
                )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    @print_info
    def test_02_signature_test_with_positive_timelap(self):
        """
        Signature test with positive timelap, when signature is made on a host
        that has a positive (greater) time difference with server. In this
        particular example, the host time is 5 minutes ahead the server time.
        """
        flow = []

        datetime_timelap = Signature.datetime_to_unix_timestamp(
            datetime.datetime.now() + datetime.timedelta(seconds=300)
            )

        flow.append(('Valid until used', datetime_timelap))
        flow.append(('Valid until used (human readable)', timestap_to_human_readable(datetime_timelap)))

        # Generate signature
        sig = Signature.generate_signature(
            auth_user = self.auth_user,
            secret_key = self.secret_key,
            valid_until = datetime_timelap
            )

        flow.append(('Signature generated', sig.signature))
        flow.append(('Signature is expired', sig.is_expired()))

        # Check if not expired
        self.assertTrue(not sig.is_expired())

        # Check if valid
        validation_result = Signature.validate_signature(
                signature = sig.signature,
                auth_user = self.auth_user,
                secret_key = self.secret_key,
                valid_until = sig.valid_until,
                return_object = True
                )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    @print_info
    def test_03_signature_test_with_negative_timelap(self):
        """
        Fail test. Signature test with negative timelap, when signature is made on
        a host that has a negative (less) time difference with server. In this
        particular example, the host time is 5 minutes behind the server time,
        which exceeds the signature lifetime.
        """
        flow = []

        datetime_timelap = Signature.datetime_to_unix_timestamp(
            datetime.datetime.now() - datetime.timedelta(seconds=300)
            )

        flow.append(('Valid until used', datetime_timelap))
        flow.append(('Valid until used (human readable)', timestap_to_human_readable(datetime_timelap)))

        # Generate signature
        sig = Signature.generate_signature(
            auth_user = self.auth_user,
            secret_key = self.secret_key,
            valid_until = datetime_timelap
            )

        flow.append(('Signature generated', sig.signature))
        flow.append(('Signature is expired', sig.is_expired()))

        # Check if not expired
        self.assertTrue(sig.is_expired())

        # Check if valid
        validation_result = Signature.validate_signature(
                signature = sig.signature,
                auth_user = self.auth_user,
                secret_key = self.secret_key,
                valid_until = sig.valid_until,
                return_object = True
                )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    @print_info
    def test_04_fail_signature_test(self):
        """
        Fail signature tests.
        """
        flow = []

        validation_result = Signature.validate_signature(
            signature = 'EBS6ipiqRLa6TY5vxIvZU30FpnM=',
            auth_user = 'fakeuser',
            secret_key = 'fakesecret',
            valid_until = 1377997396.0,
            return_object = True
            )

        flow.append(('Valid until used', 1377997396.0))
        flow.append(('Valid until used (human readable)', timestap_to_human_readable(1377997396.0)))
        flow.append(('Signature generated', 'EBS6ipiqRLa6TY5vxIvZU30FpnM='))
        flow.append(('Signature is expired', True))

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    @print_info
    def test_05_fail_signature_test_related_to_changes_in_validation_result_class(self):
        """
        Fail signature tests related to tiny changes in the `ValidationResult` class.
        """
        flow = []

        validation_result = Signature.validate_signature(
            signature = 'EBS6ipiqRLa6TY5vxIvZU30FpnM=',
            auth_user = 'fakeuser',
            secret_key = 'fakesecret',
            valid_until = 1377997396.0,
            return_object = True
            )

        if PY3:
            self.assertTrue(isinstance(validation_result.reason, map))
        else:
            self.assertTrue(isinstance(validation_result.reason, list))
        self.assertTrue(isinstance(validation_result.errors, list))
        self.assertTrue(isinstance(validation_result.message, text_type))
        self.assertTrue(isinstance(' '.join(validation_result.reason), text_type))
        self.assertTrue(isinstance(' '.join(map(text_type, validation_result.errors)), text_type))

        flow.append(validation_result.message)

        self.assertTrue(error_codes.INVALID_SIGNATURE in validation_result.errors)
        self.assertTrue(error_codes.SIGNATURE_TIMESTAMP_EXPIRED in validation_result.errors)

        return flow


def parse_url_params(url):
    """
    Parses URL params.

    :param str url:
    :return dict:
    """
    data = parse_qs(urlparse(url).query)
    for k, v in data.items():
        data[k] = v[0]


    return data

class URLHelperTest(unittest.TestCase):
    """
    Tests of ``ska.URLHelper`` class.
    """
    def setUp(self):
        self.auth_user = 'user'
        self.secret_key = 'secret'

    @print_info
    def test_01_signature_to_url(self):
        """
        Signature test.
        """
        flow = []

        # Generate signature
        signature = Signature.generate_signature(
            auth_user = self.auth_user,
            secret_key = self.secret_key
            )

        request_helper = RequestHelper(
            signature_param = 'signature',
            auth_user_param = 'auth_user',
            valid_until_param = 'valid_until'
            )

        signed_endpoint_url = request_helper.signature_to_url(
            signature = signature,
            endpoint_url = 'http://dev.example.com/api/'
            )

        flow.append(('URL generated', signed_endpoint_url))

        # Now parsing back the URL params.
        request_data = parse_url_params(signed_endpoint_url)

        validation_result = request_helper.validate_request_data(data=request_data, secret_key=self.secret_key)

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    @print_info
    def test_02_signature_to_url_fail(self):
        """
        Signature test. Fail test.
        """
        flow = []

        datetime_timelap = Signature.datetime_to_unix_timestamp(
            datetime.datetime.now() - datetime.timedelta(seconds=300)
            )

        # Generate signature
        signature = Signature.generate_signature(
            auth_user = self.auth_user,
            secret_key = self.secret_key,
            valid_until = datetime_timelap
            )

        request_helper = RequestHelper(
            signature_param = 'signature',
            auth_user_param = 'auth_user',
            valid_until_param = 'valid_until'
            )

        signed_endpoint_url = request_helper.signature_to_url(
            signature = signature,
            endpoint_url = 'http://dev.example.com/api/'
            )

        flow.append(('URL generated', signed_endpoint_url))

        # Now parsing back the URL params.
        request_data = parse_url_params(signed_endpoint_url)

        validation_result = request_helper.validate_request_data(data=request_data, secret_key=self.secret_key)

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

class ShortcutsTest(unittest.TestCase):
    """
    Tests for ``sign_url``, ``signature_to_dict`` and ``validate_signed_request_data`` shortcut functions.
    """
    def setUp(self):
        self.auth_user = 'user'
        self.secret_key = 'secret'
        self.endpoint_url = 'http://e.com/api/'

    @print_info
    def test_01_sign_url_and_validate_signed_request_data(self):
        """
        Tests for ``sign_url`` and ``validate_signed_request_data`` shortcut functions.
        """
        flow = []

        signed_url = sign_url(
            auth_user = self.auth_user,
            secret_key = self.secret_key,
            url = self.endpoint_url
        )

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        validation_result = validate_signed_request_data(
            data = request_data,
            secret_key = self.secret_key
            )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    @print_info
    def test_02_sign_url_and_validate_signed_request_data_fail(self):
        """
        Fail tests for ``sign_url`` and ``validate_signed_request_data`` shortcut functions.
        """
        flow = []

        datetime_timelap = Signature.datetime_to_unix_timestamp(
            datetime.datetime.now() - datetime.timedelta(seconds=300)
            )

        flow.append(('Valid until used', datetime_timelap))
        flow.append(('Valid until used (human readable)', timestap_to_human_readable(datetime_timelap)))

        signed_url = sign_url(
            auth_user = self.auth_user,
            secret_key = self.secret_key,
            url = self.endpoint_url,
            valid_until = datetime_timelap
        )

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        validation_result = validate_signed_request_data(
            data = request_data,
            secret_key = self.secret_key
            )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    @print_info
    def test_03_signature_to_dict_and_validate_signed_request_data(self):
        """
        Tests for ``signature_to_dict`` and ``validate_signed_request_data`` shortcut functions.
        """
        flow = []

        signature_dict = signature_to_dict(
            auth_user = self.auth_user,
            secret_key = self.secret_key
        )

        flow.append(('Dictionary created', signature_dict))

        # Now validate the signature data

        validation_result = validate_signed_request_data(
            data = signature_dict,
            secret_key = self.secret_key
            )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow


class ExtraTest(unittest.TestCase):
    """
    Test for extra data.
    """
    def setUp(self):
        self.auth_user = 'user'
        self.secret_key = 'secret'
        self.endpoint_url = 'http://e.com/api/'

    def __get_signed_url(self):
        return sign_url(
            auth_user = self.auth_user,
            secret_key = self.secret_key,
            url = self.endpoint_url,
            extra = {
                'provider': 'service1.example.com',
                'first_name': 'John',
                'last_name': 'Doe',
                'email': 'john.doe@mail.example.com',
            }
            )

    @print_info
    def test_01_sign_url_and_validate_signed_request_data(self):
        """
        Tests for ``sign_url`` and ``validate_signed_request_data`` shortcut functions.
        """
        flow = []

        signed_url = self.__get_signed_url()

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        #request_data['extra'] = 'provider,first_name'

        validation_result = validate_signed_request_data(
            data = request_data,
            secret_key = self.secret_key
            )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow

    @print_info
    def test_02_sign_url_and_validate_signed_request_data_tumper_extra_keys_remove(self):
        """
        Fail tests for ``sign_url`` and ``validate_signed_request_data`` shortcut functions,
        as well as providing the additional data ``extra`` and data tumpering ``extra``
        keys (remove).
        """
        flow = []

        signed_url = self.__get_signed_url()

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        # ***************************************************************************
        # ****************************** Tumpering **********************************
        # ***************************************************************************
        tumpered_request_data = copy(request_data)

        tumpered_request_data['extra'] = 'provider,first_name'

        flow.append(('Request data', request_data))
        flow.append(('Tumpered request data', tumpered_request_data))

        validation_result = validate_signed_request_data(
            data = tumpered_request_data,
            secret_key = self.secret_key
            )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    @print_info
    def test_03_sign_url_and_validate_signed_request_data_tumper_extra_keys_add(self):
        """
        Fail tests for ``sign_url`` and ``validate_signed_request_data`` shortcut functions,
        as well as providing the additional data ``extra`` and data tumpering ``extra``
        keys (add).
        """
        flow = []

        signed_url = self.__get_signed_url()

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        request_data = parse_url_params(signed_url)

        # ***************************************************************************
        # ****************************** Tumpering **********************************
        # ***************************************************************************
        tumpered_request_data = copy(request_data)

        tumpered_request_data['extra'] += ',age'
        tumpered_request_data['age'] = 27

        flow.append(('Request data', request_data))
        flow.append(('Tumpered request data', tumpered_request_data))

        validation_result = validate_signed_request_data(
            data = tumpered_request_data,
            secret_key = self.secret_key
            )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(not validation_result.result)

        return flow

    @print_info
    def test_04_sign_url_and_validate_signed_request_data_tumper_extra_keys_add(self):
        """
        Tests for ``sign_url`` and ``validate_signed_request_data`` shortcut functions,
        as well as providing the additional data ``extra`` and data tumpering ``extra``
        keys (add) repeated params.
        """
        flow = []

        signed_url = "{0}&provider=cervice0.example.com".format(self.__get_signed_url())

        # ***************************************************************************
        # ****************************** Tumpering **********************************
        # ***************************************************************************

        flow.append(('URL generated', signed_url))

        # Now parsing back the URL params and validate the signature data
        tampered_request_data = parse_url_params(signed_url)

        # ***************************************************************************
        # ****************************** Tumpering **********************************
        # ***************************************************************************
        even_more_tumpered_request_data = copy(tampered_request_data)
        even_more_tumpered_request_data['extra'] += ',provider'

        flow.append(('Tampered request data', tampered_request_data))
        flow.append(('Even more tampered request data', even_more_tumpered_request_data))

        validation_result = validate_signed_request_data(
            data = even_more_tumpered_request_data,
            secret_key = self.secret_key
            )

        flow.append(('Signature is valid', validation_result.result))
        flow.append(('Reason not valid', validation_result.reason))

        self.assertTrue(validation_result.result)

        return flow


if __name__ == "__main__":
    # Tests
    unittest.main()



__title__ = 'ska.contrib.django.ska.views'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('login',)

from django.http import HttpResponseForbidden, HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import authenticate
from django.contrib.auth import login as auth_login
from django.contrib import messages

from ska.contrib.django.ska.settings import REDIRECT_AFTER_LOGIN
from ska.contrib.django.ska.utils import get_provider_data

def login(request):
    """
    Authenticate with `ska` token into Django.

    :param django.http.HttpRequest request:
    :return django.http.HttpResponse:
    """
    user = authenticate(request=request)
    next_url = request.GET.get('next', None)

    if not next_url:
        provider_data = get_provider_data(request.REQUEST)
        if provider_data:
            next_url = provider_data.get('REDIRECT_AFTER_LOGIN', REDIRECT_AFTER_LOGIN)

    if not next_url:
        next_url = '/'

    if user is not None:
        auth_login(request, user)
        name = user.first_name or user.username
        messages.info(request, _("Login succeeded. Welcome, {0}.").format(name))
        return HttpResponseRedirect(next_url)
    else:
        return HttpResponseForbidden(_("Authentication error!"))



__title__ = 'ska.contrib.django.ska.backends'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SkaAuthenticationBackend',)

import logging

from ska import Signature, extract_signed_request_data #validate_signed_request_data, 
from ska.helpers import get_callback_func
from ska.defaults import (
    DEFAULT_SIGNATURE_PARAM, DEFAULT_AUTH_USER_PARAM, DEFAULT_VALID_UNTIL_PARAM, DEFAULT_EXTRA_PARAM
    )
from ska.exceptions import ImproperlyConfigured, InvalidData
from ska.contrib.django.ska.utils import get_provider_data

from django.contrib.auth.models import User, UNUSABLE_PASSWORD
from django.db import IntegrityError

from ska.contrib.django.ska.settings import (
    SECRET_KEY, USER_GET_CALLBACK, USER_CREATE_CALLBACK, USER_INFO_CALLBACK,
    DB_STORE_SIGNATURES, DB_PERFORM_SIGNATURE_CHECK
    )
from ska.contrib.django.ska.models import Signature as SignatureModel

logger = logging.getLogger(__file__)

class SkaAuthenticationBackend(object):
    """
    Authentication backend.
    """
    def authenticate(self, request):
        """
        :param django.http.HttpRequest request:
        :return django.contrib.auth.models.User: Instance or None on failure.
        """
        #secret_key = get_secret_key(request.REQUEST, default=SECRET_KEY)
        provider_data = get_provider_data(request.REQUEST)
        if provider_data:
            secret_key = provider_data['SECRET_KEY']
        else:
            secret_key = SECRET_KEY

        #logger.debug('secret_key: {0}'.format(secret_key))

        #validation_result = validate_signed_request_data(
        #    data = request.REQUEST,
        #    secret_key = SECRET_KEY,
        #    signature_param = DEFAULT_SIGNATURE_PARAM,
        #    auth_user_param = DEFAULT_AUTH_USER_PARAM,
        #    valid_until_param = DEFAULT_VALID_UNTIL_PARAM
        #    )

        # If authentication failed.
        #if not validation_result.result:
        #    return None

        try:
            # If authentication/data validation failed.
            signed_request_data = extract_signed_request_data(
                data = request.REQUEST,
                secret_key = secret_key,
                signature_param = DEFAULT_SIGNATURE_PARAM,
                auth_user_param = DEFAULT_AUTH_USER_PARAM,
                valid_until_param = DEFAULT_VALID_UNTIL_PARAM,
                extra_param = DEFAULT_EXTRA_PARAM,
                validate = True,
                fail_silently = False
                )
        except (ImproperlyConfigured, InvalidData) as e:
            logger.debug(str(e))
            return None

        # Get the username from request.
        auth_user = request.REQUEST.get(DEFAULT_AUTH_USER_PARAM)
        signature = request.REQUEST.get(DEFAULT_SIGNATURE_PARAM)
        valid_until = request.REQUEST.get(DEFAULT_VALID_UNTIL_PARAM)

        # All other specific data is taken from signed request data
        email = signed_request_data.get('email', '')
        first_name = signed_request_data.get('first_name', '')
        last_name = signed_request_data.get('last_name', '')

        # Storing the signatures to database if set to be so.
        if DB_STORE_SIGNATURES:
            token = SignatureModel(
                auth_user = auth_user,
                signature = signature,
                valid_until = Signature.unix_timestamp_to_date(valid_until)
            )
            try:
                token.save()
            except IntegrityError as e:
                if DB_PERFORM_SIGNATURE_CHECK:
                    # Token has already been used. Do not authenticate.
                    return None

        # Try to get user. If doesn't exist - create.
        try:
            user = User._default_manager.get(username=auth_user)

            # User get callback
            user_get_callback = provider_data.get('USER_GET_CALLBACK', USER_GET_CALLBACK)
            if user_get_callback is not None:
                callback_func = get_callback_func(user_get_callback)
                if callback_func:
                    try:
                        callback_func(user, request=request, signed_request_data=signed_request_data)
                    except Exception as e:
                        logger.debug(str(e))

        except User.DoesNotExist as e:
            user = User._default_manager.create_user(
                username = auth_user,
                email = email,
                password = UNUSABLE_PASSWORD,
                first_name = first_name,
                last_name = last_name
                )
            user.save()

            # User create callback
            user_create_callback = provider_data.get('USER_CREATE_CALLBACK', USER_CREATE_CALLBACK)
            if user_create_callback is not None:
                callback_func = get_callback_func(user_create_callback)
                if callback_func:
                    try:
                        callback_func(user, request=request, signed_request_data=signed_request_data)
                    except Exception as e:
                        logger.debug(str(e))

        # User info callback
        user_info_callback = provider_data.get('USER_INFO_CALLBACK', USER_INFO_CALLBACK)
        if user_info_callback is not None:
            callback_func = get_callback_func(user_info_callback)
            if callback_func:
                try:
                    callback_func(user, request=request, signed_request_data=signed_request_data)
                except Exception as e:
                    logger.debug(str(e))

        return user

    def get_user(self, user_id):
        """
        Get user in the ``django.contrib.auth.models.User`` if exists.

        :param int user_id:
        :return django.contrib.auth.models.User:
        """
        try:
            return User._default_manager.get(pk=user_id)
        except User.DoesNotExist as e:
            return None



from __future__ import absolute_import

"""
- `UNAUTHORISED_REQUEST_ERROR_MESSAGE` (str): Plain text error message. Defaults to "Unauthorised request. {0}".
- `UNAUTHORISED_REQUEST_ERROR_TEMPLATE` (str): Path to 401 template that should be rendered in case of 401
  responses. Defaults to empty string (not provided).
- `AUTH_USER` (str): Default ``auth_user`` for ``ska.sign_url`` function. Defaults to "ska-auth-user".
- `SECRET_KEY` (str): The shared secret key. Should be defined in `settings` module as ``SKA_SECRET_KEY``.
- `USER_GET_CALLBACK` (str): User get callback (when user is fetched in auth backend).
- `USER_CREATE_CALLBACK` (str): User create callback (when user is created in auth backend).
- `USER_INFO_CALLBACK` (str): User info callback.
- `REDIRECT_AFTER_LOGIN` (str): Redirect after login.
- `DB_STORE_SIGNATURES` (bool): If set to True, signatures are stored in the database.
- `DB_PERFORM_SIGNATURE_CHECK` (bool): If set to True, an extra check is fired on whether the token has
  already been used or not.
- `PROVIDERS` (dict): A dictionary where key is the provider UID and the key is another dictionary holding
  the following provider specific keys: 'SECRET_KEY', 'USER_GET_CALLBACK', 'USER_CREATE_CALLBACK',
  'USER_INFO_CALLBACK', 'REDIRECT_AFTER_LOGIN'. Note, that the 'SECRET_KEY' is a required key. The rest
  are optional, and if given, override respectively the values of ``ska.contrib.django.ska.settings``.
"""

__title__ = 'ska.contrib.django.ska.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'UNAUTHORISED_REQUEST_ERROR_MESSAGE', 'UNAUTHORISED_REQUEST_ERROR_TEMPLATE', 'AUTH_USER', 'SECRET_KEY',
    'USER_GET_CALLBACK', 'USER_CREATE_CALLBACK', 'USER_INFO_CALLBACK', 'REDIRECT_AFTER_LOGIN',
    'DB_STORE_SIGNATURES', 'DB_PERFORM_SIGNATURE_CHECK', 'PROVIDERS'
)

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from ska.exceptions import ImproperlyConfigured
from ska.contrib.django.ska.conf import get_setting

UNAUTHORISED_REQUEST_ERROR_MESSAGE = get_setting('UNAUTHORISED_REQUEST_ERROR_MESSAGE')
UNAUTHORISED_REQUEST_ERROR_TEMPLATE = get_setting('UNAUTHORISED_REQUEST_ERROR_TEMPLATE')
AUTH_USER = get_setting('AUTH_USER')

try:
    SECRET_KEY = settings.SKA_SECRET_KEY
except:
    raise ImproperlyConfigured(_("You should define a variable ``SKA_SECRET_KEY`` in your `settings` module!"))

USER_GET_CALLBACK = get_setting('USER_GET_CALLBACK')
USER_CREATE_CALLBACK = get_setting('USER_CREATE_CALLBACK')
USER_INFO_CALLBACK = get_setting('USER_INFO_CALLBACK')
REDIRECT_AFTER_LOGIN = get_setting('REDIRECT_AFTER_LOGIN')

DB_STORE_SIGNATURES = get_setting('DB_STORE_SIGNATURES')
DB_PERFORM_SIGNATURE_CHECK = get_setting('DB_PERFORM_SIGNATURE_CHECK')

PROVIDERS = get_setting('PROVIDERS')

def validate_providers():
    """
    Validates the providers set in projets' ``settings`` module.
    """
    for uid, data in PROVIDERS.items():
        if not 'SECRET_KEY' in data:
            raise ImproperlyConfigured(_("You should defined a key ``SECRET_KEY`` for each provider in "
                                         "your `settings module`!"))

validate_providers()



__title__ = 'ska.contrib.django.ska.admin'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SignatureAdmin',)

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from ska.contrib.django.ska.models import Signature

class SignatureAdmin(admin.ModelAdmin):
    """
    Signature admin.
    """
    list_display = ('signature', 'auth_user', 'valid_until', 'created',)
    readonly_fields = ('created',)
    list_filter = ('auth_user',)
    fieldsets = (
        (None, {
            'fields': ('signature', 'auth_user', 'valid_until')
        }),
        (_('Additional'), {
            'classes': ('collapse',),
            'fields': ('created',)
        }),
    )

    class Meta:
        app_label = _('Signature')


admin.site.register(Signature, SignatureAdmin)



__title__ = 'ska.contrib.django.ska.exceptions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
#__all__ = ('',)




import os
from setuptools import setup, find_packages

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
    readme = ''

version = '1.4.3'

setup(
    name = 'ska',
    version = version,
    description = ("Sign- and validate- data (dictionaries, strings) using symmetric-key algorithm."),
    long_description = readme,
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Topic :: Security :: Cryptography",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
    ],
    keywords = 'sign data, sign (HTTP) request, symmetric-key algorithm encryption, sign URL, python, django, '
               'password-less login django, password-less authentication backend django',
    author = 'Artur Barseghyan',
    author_email = 'artur.barseghyan@gmail.com',
    url = 'https://github.com/barseghyanartur/ska',
    package_dir = {'':'src'},
    packages = find_packages(where='./src'),
    include_package_data = True,
    install_requires = [
        'six>=1.1.0',
    ]
)



__title__ = 'ska'
__version__ = '1.4.2'
__build__ = 0x00000E
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'sign_url', 'signature_to_dict', 'validate_signed_request_data', 'extract_signed_request_data',
    'Signature', 'RequestHelper', 'SignatureValidationResult',
    )

import datetime
import time
import hmac
from base64 import b64decode, b64encode
from hashlib import sha1

from six import PY3, text_type

try:
    from six.moves.urllib.parse import urlencode
except ImportError as e:
    if PY3:
        from urllib.parse import urlencode
    else:
        from urllib import urlencode

from ska.helpers import (
    dict_keys, dict_to_ordered_list, sorted_urlencode, extract_signed_data as extract_signed_data
    )
from ska.exceptions import InvalidData, ImproperlyConfigured
from ska import error_codes
from ska.defaults import (
    SIGNATURE_LIFETIME, TIMESTAMP_FORMAT, DEFAULT_URL_SUFFIX,
    DEFAULT_SIGNATURE_PARAM, DEFAULT_AUTH_USER_PARAM, DEFAULT_VALID_UNTIL_PARAM,
    DEFAULT_EXTRA_PARAM
    )

_ = lambda x: x # For future integrations with gettext

# *******************************************************************************************************
# ******************************************* Signature *************************************************
# *******************************************************************************************************

class SignatureValidationResult(object):
    """
    Signature validation result container.

    If signature validation result is True, things like this would work:

    >>> res = SignatureValidationResult(result=True)
    >>> print bool(res)
    True
    >>> res = SignatureValidationResult(result=False, reason=[error_codes.INVALID_SIGNATURE,])
    >>> print bool(res)
    False
    """
    #__slots__ = ('result', 'reason', 'errors')

    def __init__(self, result, errors=[]):
        self.result = result
        self.errors = errors

    def __str__(self):
        return str(self.result)
    __unicode__ = __str__
    __repr__ = __str__

    def __bool__(self):
        return self.result
    __nonzero__ = __bool__

    @property
    def message(self):
        """
        Human readable message of all errors.

        :return string:
        """
        return ' '.join(map(text_type, self.errors))

    @property
    def reason(self):
        """
        For backwards compatibility. Returns list of text messages.

        :return list:
        """
        return map(text_type, self.errors)


class Signature(object):
    """
    Signature generation and validation based on symmetric keys.

    :param str signature:
    :param str auth_user:
    :param float|str valid_until:
    """
    __slots__ = ('signature', 'auth_user', 'valid_until', 'extra')

    def __init__(self, signature, auth_user, valid_until, extra={}):
        self.signature = signature
        self.auth_user = auth_user
        self.valid_until = valid_until
        self.extra = extra

    def __str__(self):
        return self.signature
    __unicode__ = __str__
    __repr__ = __str__

    def __bool__(self):
        return self.result
    __nonzero__ = __bool__

    @classmethod
    def validate_signature(cls, signature, auth_user, secret_key, valid_until, extra={}, return_object=False):
        """
        Validates the signature.

        :param str signature:
        :param str auth_user:
        :param str secret_key:
        :param float|str valid_until: Unix timestamp.
        :param dict extra: Extra arguments to be validated.
        :param bool return_object: If set to True, an instance of ``SignatureValidationResult`` is returned.
        :return bool:

        :example:
        >>> Signature.validate_signature(
            'EBS6ipiqRLa6TY5vxIvZU30FpnM=',
            'user',
            'your-secret-key',
            '1377997396.0'
            )
        False
        """
        if isinstance(signature, str):
            signature = signature.encode()

        sig = cls.generate_signature(
            auth_user = auth_user,
            secret_key = secret_key,
            valid_until = valid_until,
            extra = extra
            )

        if not return_object:
            return sig.signature == signature and not sig.is_expired()

        else:
            result = (sig.signature == signature and not sig.is_expired())
            errors = []
            if sig.signature != signature:
                errors.append(error_codes.INVALID_SIGNATURE)
            if sig.is_expired():
                errors.append(error_codes.SIGNATURE_TIMESTAMP_EXPIRED)

            return SignatureValidationResult(result, errors)

    def is_expired(self):
        """
        Checks if current signature is expired. Returns True if signature is expired and False otherwise.

        :return bool:

        :example:
        >>> sig = Signature.generate_signature('user', 'your-secret-key') # Generating the signature
        >>> sig.is_expired()
        False
        """
        now = datetime.datetime.now()
        valid_util = Signature.unix_timestamp_to_date(self.valid_until)

        # Expires > now is a valid condition here.
        res = valid_util > now

        # But we actually check agains is expired, so it's the opposite.
        return not res

    @classmethod
    def get_base(cls, auth_user, timestamp, extra={}):
        """
        Add something here so that timestamp to signature conversion is not that obvious.

        :param string auth_user:
        :param int timestamp:
        :param dict extra:
        """
        l = [str(timestamp), auth_user]

        if extra:
            urlencoded_extra = sorted_urlencode(extra)
            if urlencoded_extra:
                l.append(urlencoded_extra)

        return ("_".join(l)).encode()

    @staticmethod
    def make_secret_key(secret_key):
        """
        The secret key how its' supposed to be used in generate signature.

        :param str secret_key:
        :return str:
        """
        return secret_key.encode() #return b64encode(secret_key)

    @classmethod
    def generate_signature(cls, auth_user, secret_key, valid_until=None, lifetime=SIGNATURE_LIFETIME, extra={}):
        """
        Generates the signature. If timestamp is given, the signature is created using the given timestamp. Otherwise
        current time is used.

        :param str auth_user:
        :param str secret_key:
        :param float|str valid_until: Unix timestamp, valid until.
        :param int lifetime: Lifetime of the signature in seconds.
        :param dict extra: Additional variables to be added.
        :return str:

        :example:
        >>> sig = Signature.generate_signature('user', 'your-secret-key')
        EBS6ipiqRLa6TY5vxIvZU30FpnM=
        """
        if not valid_until:
            valid_until = time.mktime(
                (datetime.datetime.now() + datetime.timedelta(seconds=lifetime)).timetuple()
                )
        else:
            try:
                cls.unix_timestamp_to_date(valid_until)
            except Exception as e:
                return None # Something went wrong

        raw_hmac = hmac.new(
            Signature.make_secret_key(secret_key),
            cls.get_base(auth_user, valid_until, extra=extra),
            sha1
            ).digest()

        signature = b64encode(raw_hmac)

        return Signature(signature=signature, auth_user=auth_user, valid_until=valid_until, extra=extra)

    @staticmethod
    def datetime_to_timestamp(dt):
        """
        Human readable datetime according to the format specified in ``TIMESTAMP_FORMAT``.

        :param datetime.datetime dt:
        :return str:
        """
        try:
            return dt.strftime(TIMESTAMP_FORMAT)
        except Exception as e:
            pass

    @staticmethod
    def datetime_to_unix_timestamp(dt):
        """
        Converts ``datetime.datetime`` to Unix timestamp.

        :param datetime.datetime dt:
        :return float: Unix timestamp.
        """
        try:
            return time.mktime(dt.timetuple())
        except Exception as e:
            pass

    @classmethod
    def timestamp_to_date(cls, timestamp, fail_silently=True):
        """
        Converts the given timestamp to date. If ``fail_silently`` is set to False, raises
        exceptions if timestamp is not valid timestamp (according to the format we have
        specified in the ``TIMESTAMP_FORMAT``). Mainly used internally.

        :param str timestamp:
        :param bool fail_silently:
        :return str:
        """
        try:
            return datetime.datetime.strptime(timestamp, TIMESTAMP_FORMAT)
        except Exception as e:
            if fail_silently is not True:
                raise e
            else:
                return None

    @classmethod
    def unix_timestamp_to_date(cls, timestamp, fail_silently=True):
        """
        Converts the given Unix timestamp to date. If ``fail_silently`` is set to False, raises
        exceptions if timestamp is not valid timestamp.

        :param float|str timestamp: UNIX timestamp. Parsable to float.
        :param bool fail_silently:
        :return str:
        """
        try:
            return datetime.datetime.fromtimestamp(float(timestamp))
        except Exception as e:
            if fail_silently is not True:
                raise e
            else:
                return None

# *******************************************************************************************************
# **************************************** Request helper ***********************************************
# *******************************************************************************************************

class RequestHelper(object):
    """
    Request helper for easy put/extract of signature params from URLs.

    :param str signature_param:
    :param str auth_user_param:
    :param str valid_until_param:
    :param str extra_param:
    """
    def __init__(self, signature_param=DEFAULT_SIGNATURE_PARAM, auth_user_param=DEFAULT_AUTH_USER_PARAM,
                 valid_until_param=DEFAULT_VALID_UNTIL_PARAM, extra_param=DEFAULT_EXTRA_PARAM):
        self.signature_param = signature_param
        self.auth_user_param = auth_user_param
        self.valid_until_param = valid_until_param
        self.extra_param = extra_param

    def signature_to_url(self, signature, endpoint_url='', suffix=DEFAULT_URL_SUFFIX):
        """
        URL encodes the signature params.

        :param ska.Signature signature:
        :param str endpoint_url:
        :param str suffix: Suffix to add after the ``endpoint_url`` and before the appended
            signature params.
        :return str:

        :example:

        Required imports.

        >>> from ska import Signature, RequestHelper

        Generate signature.

        >>> signature = Signature.generate_signature(
        >>>     auth_user = 'user',
        >>>     secret_key = 'your-secret-key'
        >>>     )

        Create a request helper.

        >>> request_helper = RequestHelper(
        >>>     signature_param = 'signature',
        >>>     auth_user_param = 'auth_user',
        >>>     valid_until_param = 'valid_until'
        >>> )

        Appending signature params to the endpoint URL.

        >>> url = request_helper.signature_to_url(
        >>>     signature = signature,
        >>>     endpoint_url = 'http://e.com/api/'
        >>> )
        http://e.com/api/?valid_until=1378045287.0&auth_user=user&signature=YlZpLFsjUKBalL4x5trhkeEgqE8%3D
        """
        params = {
            self.signature_param: signature.signature,
            self.auth_user_param: signature.auth_user,
            self.valid_until_param: signature.valid_until,
            self.extra_param: dict_keys(signature.extra, return_string=True),
        }

        # Make some check that params used do not overlap with names reserved (`auth_user`, `signature`, etc).
        params.update(signature.extra)

        return "{0}{1}{2}".format(endpoint_url, suffix, urlencode(params))

    def signature_to_dict(self, signature):
        """
        Puts signature into a dictionary, which can later on be used to send when sending
        (POST) requests to the server.

        :param ska.Signature signature:
        :return dict:

        :example:

        Required imports.

        >>> from ska import Signature, RequestHelper

        Generate signature.

        >>> signature = Signature.generate_signature(
        >>>     auth_user = 'user',
        >>>     secret_key = 'your-secret-key'
        >>>     )

        Create a request helper.

        >>> request_helper = RequestHelper(
        >>>     signature_param = 'signature',
        >>>     auth_user_param = 'auth_user',
        >>>     valid_until_param = 'valid_until'
        >>> )

        Appending signature params to the endpoint URL.

        >>> signed_dict = request_helper.signature_to_dict(
        >>>     signature = signature
        >>> )
        {
            'signature': 'YlZpLFsjUKBalL4x5trhkeEgqE8=',
            'auth_user': 'user',
            'valid_until': '1378045287.0'
        }
        """
        d = {
            self.signature_param: signature.signature,
            self.auth_user_param: signature.auth_user,
            self.valid_until_param: signature.valid_until,
            self.extra_param: dict_keys(signature.extra, return_string=True),
        }

        d.update(signature.extra)

        return d

    def validate_request_data(self, data, secret_key):
        """
        Validates the request data.
        
        :param dict data:
        :param str secret_key:
        :return ska.SignatureValidationResult:

        :example:
        If your imaginary ``HttpRequest`` object has ``GET`` property (dict), then you
        would validate the request data as follows.

        Create a ``RequestHelper`` object with param names expected.

        Required imports.

        >>> from ska import RequestHelper

        Create a request helper.

        >>> request_helper = RequestHelper(
        >>>     signature_param = 'signature',
        >>>     auth_user_param = 'auth_user',
        >>>     valid_until_param = 'valid_until'
        >>> )

        Validate the request data.
        
        >>> validation_result = request_helper.validate_request_data(
        >>>     data = request.GET,
        >>>     secret_key = 'your-secret-key'
        >>> )
        """
        signature = data.get(self.signature_param, '')
        auth_user = data.get(self.auth_user_param, '')
        valid_until = data.get(self.valid_until_param, '')

        extra = extract_signed_data(data=data, extra=data.get(self.extra_param, '').split(','))

        validation_result = Signature.validate_signature(
            signature = signature,
            auth_user = auth_user,
            secret_key = secret_key,
            valid_until = valid_until,
            return_object = True,
            extra = extra
            )

        return validation_result

    def extract_signed_data(self, data, secret_key=None, validate=False, fail_silently=False):
        """
        Extracts signed data from the request.
        """
        if validate:
            if not secret_key:
                if fail_silently:
                    return {}
                raise ImproperlyConfigured("You should provide `secret_key` if `validate` is set to True.")
            validation_result = self.validate_request_data(data, secret_key)
            if not validation_result.result:
                if fail_silently:
                    return {}
                raise InvalidData(validation_result.message)

        return extract_signed_data(data=data, extra=data.get(self.extra_param, '').split(','))

# *******************************************************************************************************
# ************************************ Shortcut functions ***********************************************
# *******************************************************************************************************

def sign_url(auth_user, secret_key, valid_until=None, lifetime=SIGNATURE_LIFETIME, url='', \
             suffix=DEFAULT_URL_SUFFIX, signature_param=DEFAULT_SIGNATURE_PARAM, \
             auth_user_param=DEFAULT_AUTH_USER_PARAM, valid_until_param=DEFAULT_VALID_UNTIL_PARAM, \
             extra={}, extra_param=DEFAULT_EXTRA_PARAM):
    """
    Signs the URL.

    :param str auth_user: Username of the user making the request.
    :param str secret_key: The shared secret key.
    :param float|str valid_until: Unix timestamp. If not given, generated automatically (now + lifetime).
    :param int lifetime: Signature lifetime in seconds.
    :param str url: URL to be signed.
    :param str suffix: Suffix to add after the ``endpoint_url`` and before the appended signature params.
    :param str signature_param: Name of the GET param name which would hold the generated signature value.
    :param str auth_user_param: Name of the GET param name which would hold the ``auth_user`` value.
    :param str valid_until_param: Name of the GET param name which would hold the ``valid_until`` value.
    :param dict extra: Extra variables to add to the request.
    :param str extra_param: Name of the GET param name which would hold the ``extra_keys`` value.
    :return str:

    :example:
    Required imports.

    >>> from ska import sign_url

    Producing a signed URL.

    >>> signed_url = sign_url(
    >>>     auth_user='user', secret_key='your-secret_key', lifetime=120, \
    >>>     url='http://e.com/api/', signature_param=DEFAULT_SIGNATURE_PARAM,
    >>>     auth_user_param=DEFAULT_AUTH_USER_PARAM, valid_until_param=DEFAULT_VALID_UNTIL_PARAM,
    >>>     extra = {'provider': 'service1.example.com', 'email': 'john.doe@mail.example.com'},
    >>>     extra_param = DEFAULT_EXTRA_PARAM
    >>> )
    http://e.com/api/?valid_until=1378045287.0&auth_user=user&signature=YlZpLFsjUKBalL4x5trhkeEgqE8%3D
    """
    if lifetime is None:
        lifetime=SIGNATURE_LIFETIME

    assert isinstance(lifetime, int)

    signature = Signature.generate_signature(
        auth_user = auth_user,
        secret_key = secret_key,
        valid_until = valid_until,
        lifetime = lifetime,
        extra = extra
        )

    request_helper = RequestHelper(
        signature_param = signature_param,
        auth_user_param = auth_user_param,
        valid_until_param = valid_until_param,
        extra_param = extra_param
    )

    signed_url = request_helper.signature_to_url(
        signature = signature,
        endpoint_url = url,
        suffix = suffix,
    )

    return signed_url

def signature_to_dict(auth_user, secret_key, valid_until=None, lifetime=SIGNATURE_LIFETIME, \
                      signature_param=DEFAULT_SIGNATURE_PARAM, auth_user_param=DEFAULT_AUTH_USER_PARAM, \
                      valid_until_param=DEFAULT_VALID_UNTIL_PARAM, extra={}, \
                      extra_param=DEFAULT_EXTRA_PARAM):
    """
    Returns a dictionary containing the signature data params.

    :param str auth_user: Username of the user making the request.
    :param str secret_key: The shared secret key.
    :param float|str valid_until: Unix timestamp. If not given, generated
        automatically (now + lifetime).
    :param int lifetime: Signature lifetime in seconds.
    :param str signature_param: Name of the (for example POST) param name which would hold the
        generated ``signature`` value.
    :param str auth_user_param: Name of the (for example POST) param name which would hold
        the ``auth_user`` value.
    :param str valid_until_param: Name of the (for example POST) param name which would hold
        the ``valid_until`` value.
    :param dict extra: Additional arguments for the signature.
    :param str extra_param: Name of the (for example POST) param name which would hold
        the ``extra`` keys value.
    :return str:

    :example:
    Required imports.

    >>> from ska import signature_to_dict

    Producing a dictionary with signature data.

    >>> signature_dict = signature_to_dict(
    >>>     auth_user='user', secret_key='your-secret_key', lifetime=120, \
    >>>     signature_param=DEFAULT_SIGNATURE_PARAM, auth_user_param=DEFAULT_AUTH_USER_PARAM, \
    >>>     valid_until_param=DEFAULT_VALID_UNTIL_PARAM
    >>> )
    {
        'signature': 'YlZpLFsjUKBalL4x5trhkeEgqE8=',
        'auth_user': 'user',
        'valid_until': '1378045287.0'
    }
    """
    if lifetime is None:
        lifetime=SIGNATURE_LIFETIME

    assert isinstance(lifetime, int)

    signature = Signature.generate_signature(
        auth_user = auth_user,
        secret_key = secret_key,
        valid_until = valid_until,
        lifetime = lifetime,
        extra = extra
        )

    request_helper = RequestHelper(
        signature_param = signature_param,
        auth_user_param = auth_user_param,
        valid_until_param = valid_until_param,
        extra_param = extra_param
    )

    signature_dict = request_helper.signature_to_dict(
        signature = signature
    )

    return signature_dict

def validate_signed_request_data(data, secret_key, signature_param=DEFAULT_SIGNATURE_PARAM, \
                                 auth_user_param=DEFAULT_AUTH_USER_PARAM, \
                                 valid_until_param=DEFAULT_VALID_UNTIL_PARAM, \
                                 extra_param=DEFAULT_EXTRA_PARAM):
    """
    Validates the signed request data.

    :param dict data: Dictionary holding the (HTTP) request (for example GET or POST) data.
    :param str secret_key: The shared secret key.
    :param str signature_param: Name of the (for example GET or POST) param name which holds
        the ``signature`` value.
    :param str auth_user_param: Name of the (for example GET or POST) param name which holds
        the ``auth_user`` value.
    :param str valid_until_param: Name of the (foe example GET or POST) param name which holds
        the ``valid_until`` value.
    :param str extra_param: Name of the (foe example GET or POST) param name which holds
        the ``extra`` keys value.
    :return ska.SignatureValidationResult: A ``ska.SignatureValidationResult`` object with the
        following properties:
            - `result` (bool): True if data is valid. False otherwise.
            - `reason` (list): List of strings, indicating validation errors. Empty list in case
              if `result` is True.
    """
    request_helper = RequestHelper(
        signature_param = signature_param,
        auth_user_param = auth_user_param,
        valid_until_param = valid_until_param,
        extra_param = extra_param
    )

    validation_result = request_helper.validate_request_data(
        data = data,
        secret_key = secret_key
    )

    return validation_result

def extract_signed_request_data(data, secret_key=None, signature_param=DEFAULT_SIGNATURE_PARAM, \
                                auth_user_param=DEFAULT_AUTH_USER_PARAM, \
                                valid_until_param=DEFAULT_VALID_UNTIL_PARAM, \
                                extra_param=DEFAULT_EXTRA_PARAM, validate=False, fail_silently=False):
    """
    Validates the signed request data.

    :param dict data: Dictionary holding the (HTTP) request (for example GET or POST) data.
    :param str secret_key: The shared secret key.
    :param str signature_param: Name of the (for example GET or POST) param name which holds
        the ``signature`` value.
    :param str auth_user_param: Name of the (for example GET or POST) param name which holds
        the ``auth_user`` value.
    :param str valid_until_param: Name of the (foe example GET or POST) param name which holds
        the ``valid_until`` value.
    :param str extra_param: Name of the (foe example GET or POST) param name which holds
        the ``extra`` value.
    :param bool validate: If set to True, request data is validated before returning the result.
    :param bool fail_silently: If set to True, exceptions are ommitted.
    :return dict: Dictionary with signed request data.
    """
    request_helper = RequestHelper(
        signature_param = signature_param,
        auth_user_param = auth_user_param,
        valid_until_param = valid_until_param,
        extra_param = extra_param
    )

    return request_helper.extract_signed_data(
        data,
        secret_key = secret_key,
        validate = validate,
        fail_silently = fail_silently
        )
